/*
   Tabulator width: 3
   Comments from: Column 60                                 *Column 60*
*/

/*----------------------------------------------------------------------------------------*/ 
/* Global includes                                                                        */
/*----------------------------------------------------------------------------------------*/ 
#include <PORTAB.H>
#include <TOS.H>
#include <VDI.H>
#include <MT_AES.H>

#include <string.h>

#include <stdio.h>

/*----------------------------------------------------------------------------------------*/ 
/* Local includes                                                                         */
/*----------------------------------------------------------------------------------------*/ 
#include "ADAPTRSC.H"

/*----------------------------------------------------------------------------------------*/ 
/*                                                                                        */
/*----------------------------------------------------------------------------------------*/ 

#define  contrl   (_GemParBlk.contrl)
#define  global   (_GemParBlk.global)
#define  intin    (_GemParBlk.intin)
#define  intout   (_GemParBlk.intout)
#define  addrin   (_GemParBlk.addrin)
#define  addrout  (_GemParBlk.addrout)

#ifndef  __MTAES__

#ifndef WHITEBAK
#define  WHITEBAK 0x0040
#endif

#ifndef FL3DMASK
#define  FL3DMASK 0x0600
#endif

#ifndef FL3DIND
#define  FL3DIND  0x0200
#endif

#ifndef AD3DVALUE
#define  AD3DVALUE   6
#endif

static AESPB   pb = 
{
   contrl,
   global,
   intin,
   intout,
   (int *) addrin,                                          /* Stupid typecast... */
   (int *) addrout
};

#endif

typedef struct
{
   LONG  id;
   LONG  value;
} COOKIE;

typedef  void  *DOSVARS;                                    /* Is not used in this source */

typedef struct
{
   LONG  magic;                                             /* Must be $87654321 */
   void  *membot;                                           /* End of the AES variables */
   void  *aes_start;                                        /* Start address */

   LONG  magic2;                                            /* It's 'MAGX' */
   LONG  date;                                              /* Creation date: ddmmyyyy */
   void  (*chgres)( WORD res, WORD txt );                   /* Change resolution */
   LONG  (**shel_vector)(void);                             /* ROM desktop */
   BYTE  *aes_bootdrv;                                      /* DESKTOP.INF fits here */
   WORD  *vdi_device;                                       /* Driver used by AES */
   void  *reservd1;
   void  *reservd2;
   void  *reservd3;

   int  version;                                            /* Version number */
   int  release;                                            /* Release status */

} AESVARS;

typedef struct
{
   LONG     config_status;
   DOSVARS  *dosvars;
   AESVARS  *aesvars;
} MAGX_COOKIE;


static USERBLK *substitute_ublks;
static OBJECT  *radio_slct;
static OBJECT  *radio_deslct;
static WORD radio_bgcol;
static WORD magic_version;

extern WORD vdi_handle;                                     /* Handle of the VDI workstation for userdefs */
extern WORD pwchar;                                         /* Character width (8) */
extern WORD phchar;                                         /* Character height (16) */

static WORD cdecl check_button( PARMBLK *parmblock );
static WORD cdecl radio_button( PARMBLK *parmblock );
static WORD cdecl group( PARMBLK *parmblock );
static WORD cdecl title( PARMBLK *parmblock );
static void userdef_text( WORD x, WORD y, BYTE *string );
static LONG search_magic_cookie( void );

#ifndef  __MTAES__
static WORD objc_sysvar( WORD ob_smode, WORD ob_swhich, WORD ob_sival1, WORD ob_sival2, WORD *ob_soval1, WORD *ob_soval2 );
#endif

/*----------------------------------------------------------------------------------------*/ 
/* Return information about the AES functions                                             */
/* Function result:     Diverse flags                                                     */
/* font_id:             ID of AES-font                                                    */
/* font_height:         Height of AES font (for vst_height())                             */
/* hor_3d:              Additional horizontal margin both sides for 3D objects            */
/* ver_3d:              Additional vertical margin both sides for 3D objects              */
/*----------------------------------------------------------------------------------------*/ 
WORD  get_aes_info( WORD *font_id, WORD *font_height, WORD *hor_3d, WORD *ver_3d )
{
   extern WORD vdi_handle;
   MAGX_COOKIE *magic;
   WORD  work_out[57];
   WORD  attrib[10];
   WORD  pens;
   WORD  flags;
   
   vq_extnd( vdi_handle, 0, work_out );
   vqt_attributes( vdi_handle, attrib );

   flags = 0;
   pens = work_out[13];                                     /* Number of colour pens */
   *font_id = attrib[0];                                    /* Standard font */
   *font_height = attrib[7];                                /* Standard height */
   *hor_3d = 0;
   *ver_3d = 0;
   
   if ( appl_find( "?AGI" ) == 0 )                          /* appl_getinfo() present? */
      flags |= GAI_INFO;

   if ( global[0] >= 0x0401 )                               /* At least AES 4.01? */
      flags |= GAI_INFO;

   magic = (MAGX_COOKIE *) Supexec( search_magic_cookie );
   magic_version = 0;
   
   if ( magic )                                             /* MagiC present? */
   {
      if ( magic->aesvars )                                 /* MagiC-AES active? */
      {
         magic_version = magic->aesvars->version;           /* MagiC version number */
         flags |= GAI_MAGIC + GAI_INFO;
      }
   }
      
   if ( flags & GAI_INFO )                                  /* Is appl_getinfo() present? */
   {
      WORD  ag1;
      WORD  ag2;
      WORD  ag3;
      WORD  ag4;

      if ( appl_getinfo( 0, &ag1, &ag2, &ag3, &ag4 ))       /* Subfunction 0, fonts */
      {
         *font_id = ag2;
         *font_height = ag1;
      }

      if ( appl_getinfo( 2, &ag1, &ag2, &ag3, &ag4 ) && ag3 )  /* Subfunction 2, colours */
         flags |= GAI_CICN;

      if ( appl_getinfo( 7, &ag1, &ag2, &ag3, &ag4 ))       /* Subfunction 7 */
         flags |= ag1 & 0x0f;

      if ( appl_getinfo( 12, &ag1, &ag2, &ag3, &ag4 ) && ( ag1 & 8 ))   /* AP_TERM? */
         flags |= GAI_APTERM;

      if ( appl_getinfo( 13, &ag1, &ag2, &ag3, &ag4 ))      /* Subfunction 13, objects */
      {
         if ( ag1 && ag2 )                                  /* 3D-objects and objc_sysvar() present? */
         {
            if ( objc_sysvar( 0, AD3DVALUE, 0, 0, hor_3d, ver_3d ))  /* 3D-look switched ont? */
            {
               if ( pens >= 16 )                            /* At least 16 colours? */
                  flags |= GAI_3D;
            }
         }
      }
   }
   
   return( flags );
}

/*----------------------------------------------------------------------------------------*/ 
/* Compensate horizontal and vertical enlargement and movement of 3D objects              */
/* Function result:     -                                                                 */
/* objs:                Pointer to die objects                                            */
/* no_objs:             Number of objects                                                 */
/* hor_3d:              Additional horizontal margin both sides for 3D objects            */
/* ver_3d:              Additional vertical margin both sides for 3D objects              */
/*----------------------------------------------------------------------------------------*/ 
void  adapt3d_rsrc( OBJECT *objs, UWORD no_objs, WORD hor_3d, WORD ver_3d )
{
   while ( no_objs > 0 )
   {
      if ( objs->ob_flags & FL3DIND )                       /* Indicator or activator? */
      {
         objs->ob_x += hor_3d;
         objs->ob_y += ver_3d;
         objs->ob_width -= 2 * hor_3d;
         objs->ob_height -= 2 * ver_3d;
      }
      objs++;                                               /* Next object */
      no_objs--;
   }
}

/*----------------------------------------------------------------------------------------*/ 
/* Clear 3D flags and adapt object sizes when 3D-look is switched off                     */
/* Function result:     -                                                                 */
/* objs:                Pointer to the objects                                            */
/* no_objs:             Number of objects                                                 */
/* ftext_to_fboxtext:   Flag to show that FTEXT objects are converted to FBOXTEXT         */
/*----------------------------------------------------------------------------------------*/ 
void  no3d_rsrc( OBJECT *objs, UWORD no_objs, WORD ftext_to_fboxtext )
{
   while ( no_objs > 0 )
   {
      if ( ftext_to_fboxtext )                              /* Convert FTEXT objects into FBOXTEXT? */
      {
         if (( objs->ob_type & 0xff ) == G_FTEXT )          /* FTEXT object? */
         {
            if ( objs->ob_flags & FL3DMASK )                /* With 3D-look? */
            {
               if ( objs->ob_spec.tedinfo->te_thickness == -2 )
               {
                  objs->ob_state |= OUTLINED;
                  objs->ob_spec.tedinfo->te_thickness = -1;
                  objs->ob_type = G_FBOXTEXT;
               }
            }
         }
      }

      objs->ob_flags &= ~FL3DMASK;                          /* Clear 3D-flags */
      
      objs++;                                               /* nchstes object */
      no_objs--;
   }
}

/*----------------------------------------------------------------------------------------*/ 
/* Test if an object is a USERDEF and if it replaces a G_STRING heading                   */
/* Function result:     Pointer to the string or 0L if it is not a USERDEF heading        */
/* obj:                 Pointer to the object                                             */
/*----------------------------------------------------------------------------------------*/ 
BYTE  *is_userdef_title( OBJECT *obj )
{
   if ( substitute_ublks )                                  /* Are MagiC-objects being replaced? */
   {
      if ( obj->ob_type == G_USERDEF )
      {
         USERBLK  *ublk;
         
         ublk = obj->ob_spec.userblk;
         
         if ( ublk->ub_code == title )                      /* Replaced heading? */
            return((BYTE *) ublk->ub_parm );                /* Return pointer to the string */
      }
   }
   return( 0L );
}

/*----------------------------------------------------------------------------------------*/ 
/* Replace MagiC objects by USERDEFs                                                      */
/* Function result:     -                                                                 */
/* objs:                Pointer to the objects                                            */
/* no_objs:             Number of objects                                                 */
/* aes_flags:           Information about the AES                                         */
/* rslct:               Pointer to image for selected radio button                        */
/* rdeslct:             Pointer to image for deselected radio button                      */
/*----------------------------------------------------------------------------------------*/ 
void  substitute_objects( OBJECT *objs, UWORD no_objs, WORD aes_flags, OBJECT *rslct, OBJECT *rdeslct )
{
   OBJECT   *obj;
   UWORD i;
   UWORD no_subs;
   
   if (( aes_flags & GAI_MAGIC ) && ( magic_version >= 0x0300 ))  /* MagiC-AES? */
   {
      substitute_ublks = 0L;
      return;  
   }

   obj = objs;                                              /* Pointer to the objects */
   i = no_objs;                                             /* Number of objects in whole resource */

   no_subs = 0;

   while ( i > 0 )
   {
      if (( obj->ob_state & WHITEBAK ) && ( obj->ob_state & 0x8000 ))   /* MagiC object? */
      {
         switch ( obj->ob_type & 0xff )
         {
            case  G_BUTTON:                                 /* Checkbox, radio button or group frame? */
            {
               no_subs++;
               break;
            }
            case  G_STRING:                                 /* Heading? */
            {
               if (( obj->ob_state & 0xff00 ) == 0xff00L )  /* Underlining of full length? */
                  no_subs++;
               break;
            }
         }
      }
      obj++;                                                /* Next object */
      i--;
   }

   if ( no_subs )                                           /* Are MagiC objects present? */
   {
      substitute_ublks = Malloc( no_subs * sizeof( USERBLK ));
      radio_slct = rslct;
      radio_deslct = rdeslct;
      
      if ( aes_flags & GAI_3D )                             /* 3D-look? */
         radio_bgcol = 8;
      else
         radio_bgcol = 0;
      
      if ( substitute_ublks )                               /* Memory available? */
      {
         USERBLK  *tmp;
         
         tmp = substitute_ublks;
   
         obj = objs;                                        /* Pointer to the objects */
         i = no_objs;                                       /* Number of objects in whole resource */
         
         while ( i > 0 )
         {
            WORD  type;
            UWORD state;
            
            type = obj->ob_type & 0x00ff;
            state = (UWORD) obj->ob_state;
            
            if (( state & WHITEBAK ) && ( state & 0x8000 )) /* MagiC object? */
            {
               state &= 0xff00;                             /* Only the upper byte is of interest */

               if ( aes_flags & GAI_MAGIC )                 /* Old MagiC-AES? */
               {
                  if (( type == G_BUTTON ) && ( state == 0xfe00 ))   /* Group frame? */
                  {
                     tmp->ub_parm = (LONG) obj->ob_spec.free_string; /* Pointer to the text */
                     tmp->ub_code = group;
                  
                     obj->ob_type = G_USERDEF;
                     obj->ob_flags &= ~FL3DMASK;            /* Clear 3D flags */
                     obj->ob_spec.userblk = tmp;            /* Pointer to the USERBLK */
                     
                     tmp++;
                  }
               }
               else                                         /* TOS-AES or other */
               {
                  switch ( type )
                  {
                     case  G_BUTTON:                        /* Checkbox, radio button or group frame? */
                     {
                        tmp->ub_parm = (LONG) obj->ob_spec.free_string; /* Pointer to the text */
         
                        if ( state == 0xfe00 )              /* Group frame? */
                           tmp->ub_code = group;
                        else if ( obj->ob_flags & RBUTTON ) /* Radio button? */
                           tmp->ub_code = radio_button;
                        else                                /* Check button */
                           tmp->ub_code = check_button;
                     
                        obj->ob_type = G_USERDEF;
                        obj->ob_flags &= ~FL3DMASK;         /* Clear 3D flags */
                        obj->ob_spec.userblk = tmp;         /* Pointer to the USERBLK */
                        
                        tmp++;
                        break;
                     }
                     case  G_STRING:                        /* Heading? */
                     {
                        if ( state == 0xff00 )              /* Underlining at full length? */
                        {
                           tmp->ub_parm = (LONG) obj->ob_spec.free_string; /* Pointer to the text */
                           tmp->ub_code = title;
                           obj->ob_type = G_USERDEF;
                           obj->ob_flags &= ~FL3DMASK;      /* Clear 3D flags */
                           obj->ob_spec.userblk = tmp;      /* Pointer to the USERBLK */
                           tmp++;
                        }
                        break;
                     }
                  }
               }
            }  
            obj++;                                          /* Next object */
            i--;
         }
      }
   }
}

/*----------------------------------------------------------------------------------------*/ 
/* Release memory for resource adaptation                                                 */
/* Function result:     -                                                                 */
/*----------------------------------------------------------------------------------------*/ 
void  substitute_free( void )
{
   if ( substitute_ublks )                                  /* Memory available? */
      Mfree( substitute_ublks );
      
   substitute_ublks = 0L;
}

/*----------------------------------------------------------------------------------------*/ 
/* USERDEF function for Check button                                                      */
/* Function result:     Non-updated object stati                                          */
/* parmblock:           Pointer to the parameter block structure                          */
/*----------------------------------------------------------------------------------------*/ 
static WORD cdecl check_button( PARMBLK *parmblock )
{
   WORD  rect[4];
   WORD  clip[4];
   WORD  xy[10];
   BYTE  *string;
   
   string = (BYTE *) parmblock->pb_parm;

   *(GRECT *) clip = *(GRECT *) &parmblock->pb_xc;          /* Clipping rectangle... */
   clip[2] += clip[0] - 1;
   clip[3] += clip[1] - 1;
   vs_clip( vdi_handle, 1, clip );                          /* Restrict redraws to the specified area */

   *(GRECT *) rect = *(GRECT *) &parmblock->pb_x;           /* Object rectangle... */
   rect[2] = rect[0] + phchar - 2;
   rect[3] = rect[1] + phchar - 2;

   vswr_mode( vdi_handle, 1 );                              /* Replacing */

   vsl_color( vdi_handle, 1 );                              /* Black */
   xy[0] = rect[0];
   xy[1] = rect[1];
   xy[2] = rect[2];
   xy[3] = rect[1];
   xy[4] = rect[2];
   xy[5] = rect[3];
   xy[6] = rect[0];
   xy[7] = rect[3];
   xy[8] = rect[0];
   xy[9] = rect[1];
   v_pline( vdi_handle, 5, xy );                            /* Draw black frame */

   vsf_color( vdi_handle, 0 );                              /* White */
   
   xy[0] = rect[0] + 1;
   xy[1] = rect[1] + 1;
   xy[2] = rect[2] - 1;
   xy[3] = rect[3] - 1;
   vr_recfl( vdi_handle, xy );                              /* Draw white box */

   if ( parmblock->pb_currstate & SELECTED )
   {
      parmblock->pb_currstate &= ~SELECTED;                 /* Clear bit */
      
      vsl_color( vdi_handle, 1 );                           /* Black - for the cross */
      xy[0] = rect[0] + 2;
      xy[1] = rect[1] + 2;
      xy[2] = rect[2] - 2;
      xy[3] = rect[3] - 2;
      v_pline( vdi_handle, 2, xy );
      
      xy[1] = rect[3] - 2;
      xy[3] = rect[1] + 2;
      v_pline( vdi_handle, 2, xy );
   }
   userdef_text( parmblock->pb_x + phchar + pwchar, parmblock->pb_y, string );

   return( parmblock->pb_currstate );
}

/*----------------------------------------------------------------------------------------*/ 
/* USERDEF function for radio button                                                      */
/* Function result:     Non-updated object stati                                          */
/* parmblock:           Pointer to the parameter block structure                          */
/*----------------------------------------------------------------------------------------*/ 
static WORD cdecl radio_button( PARMBLK *parmblock )
{
   BITBLK   *image;
   MFDB  src;
   MFDB  des;
   WORD  clip[4];
   WORD  xy[8];
   WORD  image_colors[2];
   BYTE  *string;

   *(GRECT *) clip = *(GRECT *) &parmblock->pb_xc;          /* Clipping rectangle... */
   clip[2] += clip[0] - 1;
   clip[3] += clip[1] - 1;
   vs_clip( vdi_handle, 1, clip );                          /* Restrict redraws to specified area */

   string = (BYTE *) parmblock->pb_parm;

   if ( parmblock->pb_currstate & SELECTED )                /* Selected? */
   {
      parmblock->pb_currstate &= ~SELECTED;                 /* Clear bit */

      image = radio_slct->ob_spec.bitblk;
   }
   else
      image = radio_deslct->ob_spec.bitblk;
      
   src.fd_addr = image->bi_pdata;
   src.fd_w = image->bi_wb * 8;
   src.fd_h = image->bi_hl;
   src.fd_wdwidth = image->bi_wb / 2;
   src.fd_stand = 0;
   src.fd_nplanes = 1;
   src.fd_r1 = 0;
   src.fd_r2 = 0;
   src.fd_r3 = 0;

   des.fd_addr = 0L;

   xy[0] = 0;
   xy[1] = 0;
   xy[2] = src.fd_w - 1;
   xy[3] = src.fd_h - 1;
   xy[4] = parmblock->pb_x;
   xy[5] = parmblock->pb_y;
   xy[6] = xy[4] + xy[2];
   xy[7] = xy[5] + xy[3];

   image_colors[0] = 1;                                     /* Black as foreground colour */
   image_colors[1] = radio_bgcol;                           /* Background colour */

   vrt_cpyfm( vdi_handle, MD_REPLACE, xy, &src, &des, image_colors );
   userdef_text( parmblock->pb_x + phchar + pwchar, parmblock->pb_y, string );

   return( parmblock->pb_currstate );
}

/*----------------------------------------------------------------------------------------*/ 
/* USERDEF function for group frames                                                      */
/* Function result:     Non-updated object stati                                          */
/* parmblock:           Pointer to the parameter block structure                          */
/*----------------------------------------------------------------------------------------*/ 
static WORD cdecl group( PARMBLK *parmblock )
{
   WORD  clip[4];
   WORD  obj[4];
   WORD  xy[12];
   BYTE  *string;

   string = (BYTE *) parmblock->pb_parm;

   *(GRECT *) &clip = *(GRECT *) &parmblock->pb_xc;         /* Clipping rectangle... */
   clip[2] += clip[0] - 1;
   clip[3] += clip[1] - 1;
   vs_clip( vdi_handle, 1, clip );                          /* Restrict redraws to specified area */

   vswr_mode( vdi_handle, MD_TRANS );
   vsl_color( vdi_handle, 1 );
   vsl_type( vdi_handle, 1 );

   *(GRECT *) obj = *(GRECT *) &parmblock->pb_x;            /* Object rectangle... */
   obj[2] += obj[0] - 1;
   obj[3] += obj[1] - 1;

   xy[0] = obj[0] + pwchar;
   xy[1] = obj[1] + phchar / 2;
   xy[2] = obj[0];
   xy[3] = xy[1];
   xy[4] = obj[0];
   xy[5] = obj[3];
   xy[6] = obj[2];
   xy[7] = obj[3];
   xy[8] = obj[2];
   xy[9] = xy[1];
   xy[10] = (WORD) ( xy[0] + strlen( string ) * pwchar );
   xy[11] = xy[1];
   
   v_pline( vdi_handle, 6, xy );

   userdef_text( obj[0] + pwchar, obj[1], string );

   return( parmblock->pb_currstate );
}

/*----------------------------------------------------------------------------------------*/ 
/* USERDEF function for heading                                                           */
/* Function result:     Non-updated object stati                                          */
/* parmblock:           Pointer to the parameter block structure                          */
/*----------------------------------------------------------------------------------------*/ 
static WORD cdecl title( PARMBLK *parmblock )
{
   WORD  clip[4];
   WORD  xy[4];
   BYTE  *string;

   string = (BYTE *) parmblock->pb_parm;

   *(GRECT *) &clip = *(GRECT *) &parmblock->pb_xc;         /* Clipping rectangle... */
   clip[2] += clip[0] - 1;
   clip[3] += clip[1] - 1;
   vs_clip( vdi_handle, 1, clip );                          /* Restrict redraws to specified area */

   vswr_mode( vdi_handle, MD_TRANS );
   vsl_color( vdi_handle, 1 );
   vsl_type( vdi_handle, 1 );

   xy[0] = parmblock->pb_x;
   xy[1] = parmblock->pb_y + parmblock->pb_h - 1;
   xy[2] = parmblock->pb_x + parmblock->pb_w - 1;
   xy[3] = xy[1];
   v_pline( vdi_handle, 2, xy );

   userdef_text( parmblock->pb_x, parmblock->pb_y, string );

   return( parmblock->pb_currstate );
}

static void userdef_text( WORD x, WORD y, BYTE *string )
{
   extern WORD aes_font;
   extern WORD aes_height;
   extern WORD vdi_handle;
   WORD  tmp;
   
   vswr_mode( vdi_handle, MD_TRANS );
   vst_font( vdi_handle, aes_font );                        /* Set fonts */
   vst_effects( vdi_handle, 0 );                            /* No effects */
   vst_alignment( vdi_handle, 0, 5, &tmp, &tmp );           /* Align at top edge of characters */
   vst_height( vdi_handle, aes_height, &tmp, &tmp, &tmp, &tmp );
   
   v_gtext( vdi_handle, x, y, string );
}

/*----------------------------------------------------------------------------------------*/ 
/* Search for MagiC cookie                                                                */
/* Function result:     Pointer to the cookie or 0L                                       */
/*----------------------------------------------------------------------------------------*/ 
static LONG search_magic_cookie( void )
{
   COOKIE   *search;
   
   search = *(COOKIE **) 0x5a0;
   
   if ( search )
   {
      while ( search->id )
      {
         if ( search->id == 'MagX' )
            return( search->value );

         search++;
      }     
   }
   return( 0L );
}

#ifndef  __MTAES__

static WORD objc_sysvar( WORD ob_smode, WORD ob_swhich, WORD ob_sival1, WORD ob_sival2, WORD *ob_soval1, WORD *ob_soval2 )
{
   contrl[0] = 48;
   contrl[1] = 4;
   contrl[3] = 0;
   
   intin[0] = ob_smode;
   intin[1] = ob_swhich;
   intin[2] = ob_sival1;
   intin[3] = ob_sival2;

   _crystal( &pb );

   *ob_soval1 = intout[1];
   *ob_soval2 = intout[2];
   
   return( intout[0] );
}

#endif

#undef   contrl
#undef   global
#undef   intin
#undef   intout
#undef   addrin
#undef   addrout
