/*
   Tabulator width: 3
   Comments from: Column 60                                 *Column 60*
*/

/*----------------------------------------------------------------------------------------*/ 
/* Global includes                                                                       */
/*----------------------------------------------------------------------------------------*/ 
#include <PORTAB.H>
#include <TOS.H>
#include <VDI.H>
#include <MT_AES.H>

#include <STRING.H>
#include <STDLIB.H>

/*----------------------------------------------------------------------------------------*/ 
/* Prototypes                                                                             */
/*----------------------------------------------------------------------------------------*/ 

void  cdecl display( WORD x, WORD y, WORD *clip_rect, LONG id, LONG pt, LONG ratio, BYTE *string );
WORD open_screen_wk( WORD aes_handle, WORD *work_out );
WORD        wlf_available( void );
/*----------------------------------------------------------------------------------------*/ 
/* Global variables                                                                       */
/*----------------------------------------------------------------------------------------*/ 

WORD  app_id;
WORD  aes_handle;
WORD  pwchar;
WORD  phchar;
WORD  pwbox;
WORD  phbox;

WORD  work_out[57];
WORD  vdi_handle;

/*----------------------------------------------------------------------------------------*/ 
/* Application-owned fonts - Sample structures                                            */
/*----------------------------------------------------------------------------------------*/ 

BYTE  test_pts[2] = { 12, 15 };                             /* 12 and 15 point are predefined */

FNTS_ITEM   calamus_sample = 
{
   0L,                                                      /* No more fonts */
   display,                                                 /* Pointer to display function */
   400001L,                                                 /* ID, must be >= 65536 for application-owned fonts */
   0,                                                       /* No index, as application-owned */
   0,                                                       /* Proportional font */
   1,                                                       /* Vector font */
   2,                                                       /* 2 predefined point sizes */
   "Calamus-example ? Italic",                              /* Full font name */
   "Calamus-example ?",                                     /* Name of font family */
   "Italic",                                                /* Name of style */
   test_pts,                                                /* Pointer to a field with the predefined point sizes */
   { 0L, 0L, 0L, 0L }                                       /* Reserved, must be 0 */
};

FNTS_ITEM   signum_sample =
{
   &calamus_sample,                                         /* Pointer to the next font */
   display,                                                 /* Pointer to display function */
   400000L,                                                 /* ID, must be >= 65536 for application-owned fonts */
   0,                                                       /* No index, as application-owned */
   1,                                                       /* Equidistant font (monospaced) */
   0,                                                       /* Bitmap font */
   2,                                                       /* 2 predefined point sizes */
   "Signum-example ? Roman",                                /* Full font name */
   "Signum-example ?",                                      /* Name of font family */
   "Roman",                                                 /* Name of style */
   test_pts,                                                /* Pointer to a field with the predefined point sizes */
   { 0L, 0L, 0L, 0L }                                       /* Reserved, must be 0 */
};

/*----------------------------------------------------------------------------------------*/ 
/* Macros                                                                                 */
/*----------------------------------------------------------------------------------------*/ 

#define  FONT_FLAGS  ( FNTS_BTMP + FNTS_OUTL + FNTS_MONO + FNTS_PROP )

#define  BUTTON_FLAGS ( FNTS_SNAME + FNTS_SSTYLE + FNTS_SSIZE + FNTS_SRATIO + FNTS_CHNAME + FNTS_CHSTYLE + FNTS_CHSIZE + FNTS_CHRATIO + FNTS_RATIO )

/*----------------------------------------------------------------------------------------*/ 
/* Open virtual screen workstation                                                        */
/* Function result:     VDI-handle or 0 (error)                                           */
/* aes_handle:          Handle of workstation used by AES                                 */
/* work_out:            Device information                                                */
/*----------------------------------------------------------------------------------------*/ 
WORD open_screen_wk( WORD aes_handle, WORD *work_out )
{
   WORD  work_in[11];
   WORD  handle;
   WORD  i;
         
   for( i = 1; i < 10 ; i++ )                               /* initialise work_in */
      work_in[i] = 1;
      
   work_in[0] = Getrez() + 2;                               /* Resolution */
   work_in[10] = 2;                                         /* Use raster coordinates */
   handle = aes_handle;

   v_opnvwk( work_in, &handle, work_out );
   
   return( handle );
}

/*----------------------------------------------------------------------------------------*/ 
/* Sample function to display application-owned user fonts                                */
/* The code is dummy code. For an application that uses Signum fonts, one would have to   */
/* call the internal output function here, for instance. The text to be output should be  */
/* left justified and output with the baseline as the reference point.                    */
/*                                                                                        */
/* Function result:     -                                                                 */
/* x:                   X-coordinate of the character string                              */
/* y:                   Y-coordinate of the character string                              */
/* clip_rect:           Pointer to clipping rectangle                                     */
/* id:                  Font ID                                                           */
/* pt:                  Height in 1/65536 point                                           */
/* ratio:               Width/height ratio in 1/65536                                     */
/* string:              Pointer to sample text                                            */
/*----------------------------------------------------------------------------------------*/ 
void  cdecl display( WORD x, WORD y, WORD clip_rect[4], LONG id, LONG pt, LONG ratio, BYTE *string )
{
   extern WORD vdi_handle;
   WORD  dummy;

   vswr_mode( vdi_handle, 2 );
   vs_clip( vdi_handle, 1, clip_rect );
   
   vst_font( vdi_handle, 1 );
   vst_height( vdi_handle, 4, &dummy, &dummy, &dummy, &dummy );
   vst_alignment( vdi_handle, 0, 0, &dummy, &dummy );
   vst_color( vdi_handle, 1 );
   v_gtext( vdi_handle, x, y, "Example for application-owned fonts..." );
}

/*----------------------------------------------------------------------------------------*/ 
/* Test whether wdlg_xx()/lbox_xx()/fnts_xx()-functions are present                       */
/* Function result:     1: Present,   0: Not present                                      */
/*----------------------------------------------------------------------------------------*/ 
WORD  wlf_available( void )
{
   if ( appl_find( "?AGI" ) == 0 )                          /* appl_getinfo() present? */
   {
      WORD  ag1;
      WORD  ag2;
      WORD  ag3;
      WORD  ag4;

      if ( appl_getinfo( 7, &ag1, &ag2, &ag3, &ag4 ))       /* Call subfunction 7  */
      {
         if (( ag1 & 7 ) == 7 )                             /* wdlg_xx()/lbox_xx()/fnts_xx() present? */
            return( 1 );
      }  
   }
   return( 0 );
}

/*----------------------------------------------------------------------------------------*/ 
/* Main program                                                                           */
/*----------------------------------------------------------------------------------------*/ 
WORD  main( void )
{
   extern WORD app_id;
   extern WORD aes_handle;
   extern WORD pwchar;
   extern WORD phchar;
   extern WORD pwbox;
   extern WORD phbox;
   
   FNT_DIALOG  *fnt_dialog;
   
   app_id = appl_init();

   if( app_id != -1 )                                       /* Registration successful? */
   {
      if ( wlf_available())                                 /* Dialog routines present? */
      {
         aes_handle = graf_handle( &pwchar, &phchar, &pwbox, &phbox );
         graf_mouse( ARROW, 0L );                              /* Switch on mouse cursor */
         vdi_handle = open_screen_wk( aes_handle, work_out );  /* Open workstation */
   
         if( vdi_handle != 0 )
         {
            fnt_dialog = fnts_create( vdi_handle, 0, FONT_FLAGS, FNTS_3D,
                                      "Was Shake'beer Your Favourite Poet?", 0L );
   
            if ( fnt_dialog )
            {
               WORD  button;
               WORD  check_boxes;
               LONG  id;
               LONG  pt;
               LONG  ratio;
   
               fnts_add( fnt_dialog, &signum_sample );         /* Add user's own fonts */
   
               id = 1;                                         /* System font */
               pt = 10L << 16;                                 /* Set 10 point */
               ratio = 1L << 16;                               /* Ratio 1/1 (bitmap fonts cannot be compressed or expanded) */
   
               button = fnts_do( fnt_dialog, BUTTON_FLAGS, id, pt, ratio, &check_boxes, &id, &pt, &ratio );
               switch ( button )
               {
                  case  FNTS_CANCEL:                           /* "Cancel" */
                  {
                     break;
                  }
                  case  FNTS_OK:                               /* "OK" */
                  {
                     /* Marked areas should be set to the passed */
                     /* attributes; the dialog must be closed. */
   
                     break;
                  }
                  case  FNTS_SET:                              /* "Set" */
                  {
                     /* Marked areas should be set to the passed */
                     /* attributes; the dialog will not be closed. */
   
                     break;
                  }
                  case  FNTS_MARK:                             /* "Mark" */
                  {
                     /* If marking is supported, all areas with the */
                     /* font <id> in the size <pt> with the width/height */
                     /* ratio <ratio> should be marked now. */      
                     /* If the program supports checkboxes for name, */
                     /* style, size and w/h ratio, then regard should */
                     /* also be paid to <check_boxes>. */
                     
                     break;
                  }
                  case  FNTS_OPT:                              /* Any desired button-text */
                  {
                     /* Here one could now open an application-specific */
                     /* dialog (better a window dialog)... */
                     /* ...for example for kerning or...*/
   
                     form_alert( 1, "[0][Here one can open any|desired dialog.         ][ OK ]" );
                     break;
                  }
               }
               fnts_remove( fnt_dialog );                      /* Remove users own fonts */
               fnts_delete( fnt_dialog, vdi_handle );          /* Release memory for font dialog */
            }
            v_clsvwk( vdi_handle );
         }
      }     
      else
         form_alert( 1, "[1][Please start the system|extension WDIALOG!][Quit]" );

      appl_exit();
   }
   return( 0 );
}
