/*
   Tabulator width: 3
   Comments from: Column 60                                 *Column 60*
*/

/*----------------------------------------------------------------------------------------*/ 
/* Global includes                                                                        */
/*----------------------------------------------------------------------------------------*/ 
#include <PORTAB.H>
#include <TOS.H>
#include <VDI.H>
#include <MT_AES.H>

#include <STRING.H>
#include <STDLIB.H>

/*----------------------------------------------------------------------------------------*/ 
/* Prototypes                                                                             */
/*----------------------------------------------------------------------------------------*/ 

WORD  open_screen_wk( WORD aes_handle, WORD *work_out );
WORD  wlf_available( void );

/*----------------------------------------------------------------------------------------*/ 
/* Global variables                                                                       */
/*----------------------------------------------------------------------------------------*/ 

WORD  app_id;
WORD  aes_handle;
WORD  pwchar;
WORD  phchar;
WORD  pwbox;
WORD  phbox;

WORD  work_out[57];
WORD  vdi_handle;

/*----------------------------------------------------------------------------------------*/ 
/* Macros                                                                                 */
/*----------------------------------------------------------------------------------------*/ 

#define  FONT_FLAGS  ( FNTS_BTMP + FNTS_OUTL + FNTS_MONO + FNTS_PROP )

#define  BUTTON_FLAGS ( FNTS_SNAME + FNTS_SSTYLE + FNTS_SSIZE + FNTS_SRATIO + FNTS_CHNAME + FNTS_CHSTYLE + FNTS_CHSIZE + FNTS_CHRATIO + FNTS_RATIO + FNTS_BSET + FNTS_BMARK )

/*----------------------------------------------------------------------------------------*/ 
/* Open virtual screen workstation                                                        */
/* Function result:     VDI-handle or 0 (error)                                           */
/* aes_handle:          Handle of workstation used by the AES                             */
/* work_out:            Device information                                                */
/*----------------------------------------------------------------------------------------*/ 
WORD open_screen_wk( WORD aes_handle, WORD *work_out )
{
   WORD  work_in[11];
   WORD  handle;
   WORD  i;
         
   for( i = 1; i < 10 ; i++ )                               /* Initialise work_in  */
      work_in[i] = 1;
      
   work_in[0] = Getrez() + 2;                               /* Resolution */
   work_in[10] = 2;                                         /* Use raster coordinates */
   handle = aes_handle;

   v_opnvwk( work_in, &handle, work_out );
   
   return( handle );
}

/*----------------------------------------------------------------------------------------*/ 
/* Test if wdlg_xx()/lbox_xx()/fnts_xx()-functions are present                            */
/* Function result:     1: Present,   0: Not present                                      */
/*----------------------------------------------------------------------------------------*/ 
WORD  wlf_available( void )
{
   if ( appl_find( "?AGI" ) == 0 )                          /* appl_getinfo() present? */
   {
      WORD  ag1;
      WORD  ag2;
      WORD  ag3;
      WORD  ag4;

      if ( appl_getinfo( 7, &ag1, &ag2, &ag3, &ag4 ))       /* Call subfunction 7  */
      {
         if (( ag1 & 7 ) == 7 )                             /* wdlg_xx()/lbox_xx()/fnts_xx() present? */
            return( 1 );
      }  
   }
   return( 0 );
}

/*----------------------------------------------------------------------------------------*/ 
/* Main program                                                                           */
/*----------------------------------------------------------------------------------------*/ 
WORD  main( void )
{
   extern WORD app_id;
   extern WORD aes_handle;
   extern WORD pwchar;
   extern WORD phchar;
   extern WORD pwbox;
   extern WORD phbox;
   
   FNT_DIALOG  *fnt_dialog;
   
   app_id = appl_init();

   if( app_id != -1 )                                       /* Registration successful? */
   {
      if ( wlf_available())                                 /* Dialog routines present? */
      {
         aes_handle = graf_handle( &pwchar, &phchar, &pwbox, &phbox );
         graf_mouse( ARROW, 0L );                              /* Switch on mouse cursor */
         vdi_handle = open_screen_wk( aes_handle, work_out );  /* Open workstation */
   
         if( vdi_handle != 0 )
         {
            WORD  no_fonts;
            
            no_fonts = work_out[10];                        /* Number of already present (system)fonts */
            no_fonts += vst_load_fonts( vdi_handle, 0 );    /* Plus the number of loaded in fonts */
            
            fnt_dialog = fnts_create( vdi_handle, no_fonts, FONT_FLAGS, FNTS_3D,
                                      "Was Shake'beer Your Favourite Poet?",
                                      "Why ?" );
   
            if ( fnt_dialog )
            {
               LONG  id;
               LONG  pt;
               LONG  ratio;
               WORD  x;
               WORD  y;
               
               id = 1;                                      /* System font */
               pt = 10L << 16;                              /* Set 10 point */
               ratio = 1L << 16;                            /* Ratio 1/1 (bitmap fonts cannot be compressed or expanded) */
               x = -1;
               y = -1;
   
               if ( fnts_open( fnt_dialog, BUTTON_FLAGS, x, y, id, pt, ratio ))
               {
                  EVNT  evnts;
                  WORD  button;
                  WORD  check_boxes;
                  WORD  cont;
                  WORD  index;
                  
                  index = 1;
                  cont = 1;                                 /* Continue... */
                  
                  while( cont )
                  {
                     EVNT_multi( MU_KEYBD + MU_BUTTON + MU_MESAG + MU_TIMER, 2, 1, 1, 0L, 0L, 2000, &evnts );
                     
                     if ( evnts.mwhich & MU_TIMER )
                     {
                        BYTE  name[36];
                                                
                        index++;
                        if ( index > no_fonts )
                           index = 1;

                        pt = ( index % 10 ) + 10;           /* Set size between 10 and 19 pt */
                        pt <<= 16;
                        
                        ratio = 10 + ( index % 10 );
                        ratio <<= 16;
                        ratio /= 10;                        /* W/H ratio between 1 and 1.9 */
                                                
                        id = vqt_name( vdi_handle, index, name );
                        fnts_update( fnt_dialog, BUTTON_FLAGS, id, pt, ratio );
                     }
                     
                     if ( fnts_evnt( fnt_dialog, &evnts, &button, &check_boxes, &id, &pt, &ratio ) == 0 )
                     {
                        switch ( button )
                        {
                           case  FNTS_CANCEL:               /* "Cancel" */
                           {
                              cont = 0;                     /* End loop */
                              break;
                           }
                           case  FNTS_OK:                   /* "OK" */
                           {
                              /* Marked areas should be set to the passed */
                              /* attributes; the dialog must be closed. */
   
                              cont = 0;                     /* End loop */
                              break;
                           }
                           case  FNTS_SET:                  /* "Set" */
                           {
                              /* Marked areas should be set to the passed */
                              /* attributes; the dialog will not be closed. */
   
                              break;
                           }
                           case  FNTS_MARK:                 /* "Mark" */
                           {
                              /* If marking is supported, all areas with the */
                              /* font <id> in the size <pt> with the width/height */
                              /* ratio <ratio> should be marked now. */      
                              /* If the program supports checkboxes for name, */
                              /* style, size and w/h ratio, then regard should */
                              /* also be paid to <check_boxes>. */
                              
                              break;
                           }
                           case  FNTS_OPT:                              /* Any desired button-text */
                           {
                              /* Here one could now open an application-specific */
                              /* dialog (better a window dialog)... */
                              /* ...for example for kerning or...*/
   
                              form_alert( 1, "[0][Here one can open any|desired dialog.         ][ OK ]" );
                              break;
                           }
                        }
                     }
                  }
            
                  fnts_close( fnt_dialog, &x, &y );         /* Close font dialog window */
               }
               fnts_delete( fnt_dialog, vdi_handle );       /* Release memory for font dialog */
            }
            vst_unload_fonts( vdi_handle, 0 );
            v_clsvwk( vdi_handle );
         }
      }     
      else
         form_alert( 1, "[1][Please start the system|extension WDIALOG!][Quit]" );

      appl_exit();
   }
   return( 0 );
}
