/*
   Tabulator width: 3
   Comments from: Column 60                                 *Column 60*
*/

/*----------------------------------------------------------------------------------------*/ 
/* Global includes                                                                        */
/*----------------------------------------------------------------------------------------*/ 
#include <PORTAB.H>
#include <TOS.H>
#include <VDI.H>
#include <VDICOL.H>                                         /* New VDI functions */
#include <MT_AES.H>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "PDSMPRSC.H"

/*----------------------------------------------------------------------------------------*/ 
/* Prototypes                                                                             */
/*----------------------------------------------------------------------------------------*/ 
static WORD is_macprn( WORD handle, WORD id );
void  print_sample( PRN_SETTINGS *settings );

PRN_SETTINGS   *read_psettings( PRN_DIALOG *prn_dialog );
WORD  save_psettings( PRN_SETTINGS *settings );

WORD  do_print_dialog( PRN_SETTINGS *settings, BYTE *document_name, WORD kind );

void  menu_selected( WORD title, WORD entry );
void  handle_keybd( WORD keycode, WORD kstate );
void  handle_message( WORD msg[8] );
void  event_loop( void );

void  init_rsrc( void );
WORD  wlfp_available( void );

/*----------------------------------------------------------------------------------------*/ 
/* Global variables                                                                       */
/*----------------------------------------------------------------------------------------*/ 

WORD  app_id;
WORD  aes_handle;
WORD  pwchar;
WORD  phchar;
WORD  pwbox;
WORD  phbox;

RSHDR    *rsh;
BYTE     **fstring_addr;
OBJECT   **tree_addr;
WORD     tree_count;
OBJECT   *menu_tree;

WORD  quit;

/*----------------------------------------------------------------------------------------*/ 
/* Information about the print dialog                                                     */
/* In a fully mature application these variables should be encapsulated in a              */
/* structure, so that things do not get too obscure                                       */
/*----------------------------------------------------------------------------------------*/ 
PRN_SETTINGS   *gprn_settings;
PRN_DIALOG     *gprn_dialog;
WORD           gprn_whdl;
WORD           gprn_x;
WORD           gprn_y;
WORD           gprn_is_print_dialog;

#define  obj_SELECTED( tree, obj )        tree[obj].ob_state |= SELECTED
#define  obj_DESELECTED( tree, obj )      tree[obj].ob_state &= ~SELECTED

/*----------------------------------------------------------------------------------------*/ 
/* Information for own subdialogs of the print dialog                                     */
/*----------------------------------------------------------------------------------------*/ 

int32 cdecl pinit_my_special1( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog );
int32 cdecl psub_my_special1( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog, int16 exit_obj );
int32 cdecl preset_my_special1( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog );

int32 cdecl pinit_my_special2( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog );
int32 cdecl preset_my_special2( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog );

typedef struct                                              /* Dialog description for create_sub_dialogs() */
{
   PDLG_INIT   init_dlg;
   PDLG_HNDL   do_dlg;
   PDLG_RESET  reset_dlg;
   int16       icon_index;
   int16       tree_index;
} SIMPLE_SUB;

PDLG_SUB *create_sub_dialogs( OBJECT **tree_addr, SIMPLE_SUB *create, int16 no_subs );
void  free_sub_dialogs( PDLG_SUB *sub_dialogs );

void  redraw_obj( PDLG_SUB *sub_dialog, int16 obj );

SIMPLE_SUB  my_subs_for_pdlg[2] =                           /* Field with dialog descriptions for create_sub_dialogs() */
{
   { pinit_my_special1, psub_my_special1, preset_my_special1, PI_MY_SPECIAL1, MY_SPECIAL_DLG1 },
   { pinit_my_special2, 0L, preset_my_special2, PI_MY_SPECIAL2, MY_SPECIAL_DLG2 }
};

/*----------------------------------------------------------------------------------------*/ 
/* Build up list for subdialogs                                                           */
/* Function result:     Pointer to list                                                   */
/* tree_addr:           Field with object tree addresses                                  */
/* create:                                                                                */
/* no_subs:             Number of subdialogs                                              */
/*----------------------------------------------------------------------------------------*/ 
PDLG_SUB *create_sub_dialogs( OBJECT **tree_addr, SIMPLE_SUB *create, int16 no_subs )
{
   PDLG_SUB *sub_dialogs;
   int16 i;

   sub_dialogs = 0L;

   for ( i = 0; i < no_subs; i++ )
   {
      PDLG_SUB *sub;

      sub = malloc( sizeof( PDLG_SUB ));
      
      if ( sub )
      {
         sub->next = 0L;                                    /* Pointer to successor */

         sub->option_flags = 0;
         sub->sub_id = -1;                                  /* ID of subdialog (is set by pdlg_add_sub_dialogs() ) */
         sub->sub_icon = tree_addr[ICON_DIALOG] + create->icon_index;
         sub->sub_tree = tree_addr[create->tree_index];     /* Pointer to the object tree of the subdialog */

         sub->dialog = 0L;                                  /* Pointer to the structure of the window dialog or 0L */
         sub->tree = 0L;                                    /* Pointer to the composite object tree */
         sub->index_offset = 0;                             /* Index offset of the subdialog */
         sub->reserved1 = 0;
         sub->reserved2 = 0;

         sub->init_dlg = create->init_dlg;                  /* Initialisation function */
         sub->do_dlg = create->do_dlg;                      /* Handling function */
         sub->reset_dlg = create->reset_dlg;                /* Reset function */
         sub->reserved3 = 0;
         
         sub->next = sub_dialogs;
         sub_dialogs = sub;
      }
      create++;                                             /* Pointer to the next description */
   }
   return( sub_dialogs );  
}

/*----------------------------------------------------------------------------------------*/ 
/* Release chained list for the subdialogs                                                */
/* Function result:     -                                                                 */
/* sub_dialogs:         List of the subdialogs                                            */
/*----------------------------------------------------------------------------------------*/ 
void  free_sub_dialogs( PDLG_SUB *sub_dialogs )
{
   while ( sub_dialogs )
   {  
      PDLG_SUB *next;

      next = sub_dialogs->next;
      free( sub_dialogs );
      sub_dialogs = next;
   }
}

/*----------------------------------------------------------------------------------------*/ 
/* Draw object in print dialog                                                            */
/* Function result:     -                                                                 */
/* sub_dialog:          Structure of the subdialog                                        */
/* obj:                 Number of the object                                              */
/*----------------------------------------------------------------------------------------*/ 
void  redraw_obj( PDLG_SUB *sub_dialog, int16 obj )
{
   OBJECT   *tree;
   GRECT    rect;

   tree = sub_dialog->tree;
   rect = *(GRECT *) &tree->ob_x;                           /* Dialog rectangle */
   
   wind_update( BEG_UPDATE );                               /* Lock screen */

   if ( sub_dialog->dialog )                                /* Window dialog? */
      wdlg_redraw( sub_dialog->dialog, &rect, obj, MAX_DEPTH );
   else                                                     /* Normal dialog */
      objc_draw( tree, obj, MAX_DEPTH, &rect );

   wind_update( END_UPDATE );                               /* Release screen */
}

/*----------------------------------------------------------------------------------------*/ 
/* Initialise subdialog                                                                   */
/* Function result:     0: Error   1: All OK                                              */
/* settings:            Printer settings                                                  */
/* sub_dialog:          Pointer to the subdialog structure                                */
/*----------------------------------------------------------------------------------------*/ 
int32 cdecl pinit_my_special1( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog )
{
   OBJECT   *tree;
   int16    offset;

   tree = sub_dialog->tree;                                 /* Pointer to the object tree */
   offset = sub_dialog->index_offset;                       /* Offset for the object indices */

   obj_SELECTED( tree, MYSP_FARBKEIL1 + offset );           /* Select object initially */

   return( 1 );
}

/*----------------------------------------------------------------------------------------*/ 
/* Handle buttons in subdialog                                                            */
/* Function result:     0: Error    1: All OK     PDLG_BUT_DEV: Change device             */
/* settings:            Printer settings                                                  */
/* sub_dialog:          Pointer to the subdialog structure                                */
/* exit_obj:            Object number                                                     */
/*----------------------------------------------------------------------------------------*/ 
int32 cdecl psub_my_special1( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog, int16 exit_obj )
{
   OBJECT   *tree;
   int16    offset;

   tree = sub_dialog->tree;                                 /* Pointer to the object tree */
   offset = sub_dialog->index_offset;                       /* Offset for the object indices */

   switch ( exit_obj - sub_dialog->index_offset )
   {
      case  MYSP_PUSH_ME1:                                  /* Example button */
      {
         /* The only "active" button in this subdialog */
         /* inverts the status of an icon */
      
         tree[PUSH_ME_ICON1 + offset].ob_state ^= SELECTED; /* Alter selection of an icon */
         redraw_obj( sub_dialog, PUSH_ME_ICON1 + offset );
         
         obj_DESELECTED( tree, exit_obj );                  /* Button normal again */
         redraw_obj( sub_dialog, exit_obj );
         break;
      }
   }
   return( 1 );
}

/*----------------------------------------------------------------------------------------*/ 
/* Read fields in a subdialog before the subdialog is changed                             */
/* Function result:     0: Error     1: All OK                                            */
/* settings:            Printer settings                                                  */
/* sub_dialog:          Pointer to the subdialog structure                                */
/*----------------------------------------------------------------------------------------*/ 
int32 cdecl preset_my_special1( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog )
{
   OBJECT   *tree;
   int16    offset;
   
   tree = sub_dialog->tree;
   offset = sub_dialog->index_offset;
   
   /* Evaluate object states */

   return( 1 );
}

/*----------------------------------------------------------------------------------------*/ 
/* Initialise subdialog                                                                   */
/* Function result:     0: Error     1: All OK                                            */
/* settings:            Printer settings                                                  */
/* sub_dialog:          Pointer to the subdialog structure                                */
/*----------------------------------------------------------------------------------------*/ 
int32 cdecl pinit_my_special2( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog )
{
   OBJECT   *tree;
   int16    offset;

   tree = sub_dialog->tree;                                 /* Pointer to the object tree */
   offset = sub_dialog->index_offset;                       /* Offset for the object indices */

   /* Set object states */

   return( 1 );
}

/* ... the second subdialog contains no "active" buttons and hence does */
/* not need the PDLG_HNDL function */

/*----------------------------------------------------------------------------------------*/ 
/* Read fields in the "Connect" subdialog before the subdialog is changed                 */
/* Function result:     0: Error     1: All OK                                            */
/* settings:            Printer settings                                                  */
/* sub_dialog:          Pointer to the subdialog strucure                                 */
/*----------------------------------------------------------------------------------------*/ 
int32 cdecl preset_my_special2( PRN_SETTINGS *settings, PDLG_SUB *sub_dialog )
{
   OBJECT   *tree;
   int16    offset;
   
   tree = sub_dialog->tree;
   offset = sub_dialog->index_offset;
   
   /* Evaluate object states */

   return( 1 );
}

/*----------------------------------------------------------------------------------------*/ 
/* Test if the driver is MACPRN.SYS                                                       */
/* Function result:     1: It is MACPRN.SYS                                               */
/* handle:              Handle of the printer                                             */
/* id:                  Driver number                                                     */
/*                                                                                        */
/* Note:                                                                                  */
/* The old MACPRN does not return the orientation correctly, although the driver          */
/* can rotate the page                                                                    */
/*----------------------------------------------------------------------------------------*/ 
static WORD is_macprn( WORD handle, WORD id )
{
   VDIPB pb;
   WORD  contrl[16];
   WORD  intin[1];
   WORD  ptsin[1];
   WORD  intout[64];
   WORD  ptsout[64];

   contrl[0] = 248;                                         /* Function number */
   contrl[1] = 0;
   contrl[3] = 1;                                           /* ID is passed */
   contrl[5] = 0;
   contrl[6] = handle;

   intin[0] = id;

   pb.contrl = contrl;
   pb.intin = intin;
   pb.ptsin = ptsin;
   pb.intout = intout;
   pb.ptsout = ptsout;

   vdi( &pb );

   if ( contrl[4] && intout[0] )                            /* Driver present? */
   {
      BYTE  macprn[6] = { 'M','A','C','P','R','N' };
      WORD  i;
      WORD  len;

      len = contrl[4];
      if ( len > 6 )
         len = 6;                                           /* Only test the first 6 characters */
      
      for ( i = 0; i < len; i++ )
      {
        if ( intout[i] != macprn[i] )
        {
          if ( intout[i] != ( macprn[i] + ( 'a' - 'A' )))
               return( 0 );                                 /* It's not MACPRN */
         }
      }
      return( 1 );
   }
   return( 0 );
}

/*----------------------------------------------------------------------------------------*/ 
/* Core function for print output                                                         */
/* Function result:     -                                                                 */
/* settings:            Printer settings                                                  */
/*----------------------------------------------------------------------------------------*/ 
void  print_sample( PRN_SETTINGS *settings )
{
#if 0
   WORD  prn_work_out[57];
   WORD  prn_handle;

   prn_handle = v_opnprn( aes_handle, settings, prn_work_out );

   if ( prn_handle )                                        /* Could the printer driver be opened? */
   {
      WORD  page_inc;
      WORD  no_copies;
      WORD  do_landscape;
      WORD  page;

#if 0
      /* 
         The metafile driver needs the following three calls so
         that programs can scale and display the metafile correctly.
         The metafile driver also differs from all other drivers in 
         so far that it does not allow any vrt/vro_cpyfm() calls.
      */

      if ( settings->driver_id == 31 )                      /* Metafile? */
      {
         v_meta_extents( prn_handle, x_min, y_min, x_max, y_max );   /* Bounding rectangle */
         vm_pagesize( prn_handle, pgwidth, pgheight );      /* Page dimensions */
         vm_coords( prn_handle, llx, lly, urx, ury );       /* Coordinate system */
      }
#endif

      if ( settings->first_page > settings->last_page )     /* Print in reverse order? */
         page_inc = -1;
      else                                                  /* Increasing page order */
         page_inc = 1;

      if ( v_copies( prn_handle, -1 ) <= 1 )                /* Can the driver not produce multiple copies ? */
         no_copies = settings->no_copies;
      else                                                  /* The driver itself prepares multiple copies */
         no_copies = 1;

      do_landscape = 0;                                     /* Driver looks after rotation */
      if ( settings->orientation == PG_LANDSCAPE )          /* Landscape format set? */
      {
         if ( v_orient( prn_handle, -1 ) != 1 )             /* Landscape format not set? */

            if ( is_macprn( prn_handle, settings->driver_id ) == 0 )
               do_landscape = 1;                            /* Rotate it yourself */
         }
      }

      for ( page = settings->first_page; page <= settings->last_page; page += page_inc )
      {
         if (((( page & 1 ) == 0 ) && ( settings->page_flags & PG_EVEN_PAGES )) ||  /* Even page numbers permitted? */
             (( page & 1 ) && ( settings->page_flags & PG_ODD_PAGES )))             /* Odd page numbers permitted? */
         {
            WORD  copy;
            
            for ( copy = 0; copy < no_copies; copy++ )      /* Loop for drivers that cannot produce multiple copies */
            {
               if ( do_landscape )                          /* Can the driver not output landscape format? */
               {
                  /* Here one would arrange output with rotated coordinate system */
               }
               else                                         /* Driver looks after landscape output */
               {
                  /* Output normally */
               }
               
               v_updwk( prn_handle );                       /* Build up page */
               v_clrwk( prn_handle );                       /* Form feed */
            }
         }
      }
   
      v_clswk( prn_handle );                                /* Close driver again */
   }     
#else
   form_alert( 1, fstring_addr[PRINT_ALERT] );
#endif
}

/*----------------------------------------------------------------------------------------*/ 
/* Returned saved printer settings                                                        */
/* Function result:     Pointer to printer settings or 0L                                 */
/* prn_dialog:          Pointer to management structure (if opened permanently) or 0L     */
/*----------------------------------------------------------------------------------------*/ 
PRN_SETTINGS   *read_psettings( PRN_DIALOG *prn_dialog )
{
   PRN_DIALOG     *p;
   PRN_SETTINGS   *settings;

   settings = 0L;

   if ( prn_dialog == 0L )                                  /* Not yet open? */
   {
      p = pdlg_create( 0 );                                 /* Open short term */
      if ( p == 0L ) 
         return( 0L );
   
      prn_dialog = p;
   }
   else
      p = 0L;

#if 0 

   If the program had saved the settings to a file, then the following  
   code should be used:
   
   settings = read_settings_from_file();                    /* Get settings from program-owned file */
   
   if ( settings )
   {
      if ( pdlg_validate_settings( prn_dialog, settings ) == 0 )  /* Serious error? */
      {
         Mfree( settings );
         settings = 0L;
      }     
   }
#endif

   if ( settings == 0L )
   {
      settings =  Malloc( pdlg_get_setsize());              /* Allocate memory */
      
      if ( settings )
         pdlg_dflt_settings( prn_dialog, settings );        /* ...and initialise */
   }
   
   if ( p )                                                 /* Only opened short term? */
      pdlg_delete( p );

   return( settings );
}

/*----------------------------------------------------------------------------------------*/ 
/* Saving printer settings                                                                */
/* Function result:     1: All OK                                                         */
/* settings:            Printer settings                                                  */
/*----------------------------------------------------------------------------------------*/ 
WORD  save_psettings( PRN_SETTINGS *settings )
{
   if ( settings )
   {
      #if 0

      If the program would save the settings to a file, the following 
      code should be used:
        
      write_settings_to_file( settings );                   /* Save printer settings */
      
      #endif
   
      Mfree( settings );                                    /* Release memory */
      return( 1 );
   }
   
   return( 0 );
}

/* Offer portrait/landscape, multiple copies, scaling and even/odd pages */   
#define  PRINT_FLAGS PDLG_ALWAYS_ORIENT + PDLG_ALWAYS_COPIES + PDLG_EVENODD

/*----------------------------------------------------------------------------------------*/ 
/* Display print dialog                                                                   */
/* Function result:     0, PDLG_CANCEL or PDLG_OK                                         */
/* settings:            Printer settings                                                  */
/* document_name:       Document name                                                     */
/* kind:                PDLG_PREFS: Settings dialog, PDLG_PRINT: Print dialog             */
/*----------------------------------------------------------------------------------------*/ 
WORD  do_print_dialog( PRN_SETTINGS *settings, BYTE *document_name, WORD kind )
{
   extern PRN_DIALOG *gprn_dialog;
   WORD  button;

   gprn_dialog = pdlg_create( PDLG_3D );                 /* Allocate memory, scan drivers */
   if ( gprn_dialog )
   {
      PDLG_SUB *sub_dialogs;
      
      sub_dialogs = create_sub_dialogs( tree_addr, my_subs_for_pdlg, 2 );  /* Buid up list for two subdialogs */
      
      if ( sub_dialogs )
         pdlg_add_sub_dialogs( gprn_dialog, sub_dialogs );  /* Add subdialogs */
      
      button = pdlg_do( gprn_dialog, settings, document_name, PRINT_FLAGS + kind );
      
      if ( sub_dialogs )
      {
         pdlg_remove_sub_dialogs( gprn_dialog );         /* Remove subdialogs */
         free_sub_dialogs( sub_dialogs );                /* Release memory for list */
      }

      pdlg_delete( gprn_dialog );                        /* Release memory */
      gprn_dialog = 0L;
      return( button );
   }

   return( 0 );
}

/*----------------------------------------------------------------------------------------*/ 
/* React to click on menu entry                                                           */
/* Function result:     -                                                                 */
/* title:               Object number of the menu title                                   */
/* entry:               Object number of the menu entry                                   */
/*----------------------------------------------------------------------------------------*/ 
void  menu_selected( WORD title, WORD entry )
{
   menu_tnormal( menu_tree, title, 0 );                     /* Select menu title */
   
   switch( entry )
   {
      case  DLAYOUT:
      {
         form_alert( 1, fstring_addr[LAYOUT_ALERT] );
         break;
      }
      case  DPAPIER:
      {
         do_print_dialog( gprn_settings, " Untitled ", PDLG_PREFS ); /* Display settings dialog */
         break;
      }
      case  DDRUCKEN:
      {
         WORD  button;

         button = do_print_dialog( gprn_settings, " Untitled ", PDLG_PRINT ); /* Display print dialog */

         if ( button == PDLG_OK )                     /* "Print" selected? */
            print_sample( gprn_settings );

         break;
      }
      case  DQUIT:
      {
         quit = 1;
         break;
      }
   }
   menu_tnormal( menu_tree, title, 1 );                     /* Deselect menu title */
}

/*----------------------------------------------------------------------------------------*/ 
/* React to keypress                                                                      */
/* Function result:     -                                                                 */
/* keycode:             Is events->key                                                    */
/* kstate:              Is events->kstate                                                 */
/*----------------------------------------------------------------------------------------*/ 
void  handle_keybd( WORD keycode, WORD kstate )
{
   if ( kstate == K_CTRL )
   {
      switch ( keycode & 0xff )
      {
         case 'P'-64:   menu_selected( MDATEI, DDRUCKEN );  break;
         case 'Q'-64:   menu_selected( MDATEI, DQUIT );  break;
      }
   }
}

/*----------------------------------------------------------------------------------------*/ 
/* Handle AES messages                                                                    */
/* Function result:    -                                                                  */
/* msg:                 Pointer to the message buffer                                     */
/*----------------------------------------------------------------------------------------*/ 
void  handle_message( WORD msg[8] )
{
   switch ( msg[0] )
   {
      case  MN_SELECTED:   menu_selected( msg[3], msg[4] ); break;
   }
}

void  event_loop( void )
{
   EVNT  events;
   
   quit = 0;
   
   while ( quit == 0 )
   {
      EVNT_multi( MU_KEYBD + MU_BUTTON + MU_MESAG, 2, 1, 1, 0L, 0L, 0L, &events );

      if ( events.mwhich & MU_KEYBD )
         handle_keybd( events.key, events.kstate );

      if ( events.mwhich & MU_MESAG )
         handle_message( events.msg );
   }
}

/*----------------------------------------------------------------------------------------*/ 
/* Initialise resource and structures belonging to it                                     */
/*----------------------------------------------------------------------------------------*/ 
void  init_rsrc( void )
{
   rsh = *((RSHDR **)(&_GemParBlk.global[7]));              /* Get address of resource header via global[7/8] */

   tree_addr = (OBJECT **)(((UBYTE *)rsh) + rsh->rsh_trindex); /* Get pointer to the object tree table */
   tree_count = rsh->rsh_ntree;                             /* ...and determine number of object trees (counting from 1) */
   fstring_addr = (BYTE **)((UBYTE *)rsh + rsh->rsh_frstr);

   menu_tree = tree_addr[MENU];

   /* Here one could call adaptrsc if required */
}

/*----------------------------------------------------------------------------------------*/ 
/* Test if wdlg_xx()/lbox_xx()/fnts_xx()/pdlg_xx()-functions are present                  */
/* Function result:     1: Present,   0: Not present                                      */
/*----------------------------------------------------------------------------------------*/ 
WORD  wlfp_available( void )
{
   if ( appl_find( "?AGI" ) == 0 )                          /* appl_getinfo() present? */
   {
      WORD  ag1;
      WORD  ag2;
      WORD  ag3;
      WORD  ag4;

      if ( appl_getinfo( 7, &ag1, &ag2, &ag3, &ag4 ))       /* Call subfunction 7 */
      {
         if (( ag1 & 0x17 ) == 0x17 )                       /* wdlg_xx()/lbox_xx()/fnts_xx()/pdlg_xx() present? */
            return( 1 );
      }  
   }
   return( 0 );
}

/*----------------------------------------------------------------------------------------*/ 
/* Main programm                                                                          */
/*----------------------------------------------------------------------------------------*/ 
WORD  main( void )
{
   extern WORD app_id;
   extern WORD aes_handle;
   extern WORD pwchar;
   extern WORD phchar;
   extern WORD pwbox;
   extern WORD phbox;
   
   app_id = appl_init();

   if( app_id != -1 )                                       /* Registration successful? */
   {
      if ( wlfp_available())                                /* Dialog routines present? */
      {
         aes_handle = graf_handle( &pwchar, &phchar, &pwbox, &phbox );
         graf_mouse( ARROW, 0L );                           /* Switch on mouse cursor */
   
         if( rsrc_load( "PDSMPRSC.RSC" ) )
         {
            init_rsrc();                                    /* Adapt resource */

            gprn_settings = read_psettings( 0L );           /* Read in/create printer settings */
            gprn_x = -1;                                    /* Centre window initially */
            gprn_y = -1;

            menu_bar( menu_tree, 1 );                       /* Display menu bar */
            event_loop();
            menu_bar( menu_tree, 0 );                       /* No longer display menu bar */

            save_psettings( gprn_settings );                /* Save priner settings */
            rsrc_free();
         }
      }
      else
         form_alert( 1, "[1][Please start the system|extension WDIALOG!][Quit]" );

      appl_exit();
   }
   return( 0 );
}
